import {Model} from "./model/model.ts";
import {ComponentChildren} from "preact";
import {Context, createContext, useContext, useEffect, useState} from "react";
import {Supervisor} from "./model/supervisor.ts";
import {TurnDuration} from "./model/lifecycle.ts";

function loadAndSuperviseModel(supervisor: Supervisor): Model {
  let model = new Model();
  model.setSupervisor(supervisor);
  let sched = (turnProgress: number) => {
    let milliseconds = TurnDuration * (1 - turnProgress);
    setTimeout(upd, milliseconds)
  }
  let upd = () => {
    model.nextTurn();
    sched(0.0);
  }
  sched(0.0);
  return model;
}

let SupervisorContext: Context<[Supervisor, Model] | undefined> = createContext(
  undefined as [Supervisor, Model] | undefined);

export function HasModel({children}: {children: ComponentChildren}) {
  let [val, setVal] =
    useState(undefined as [Supervisor, Model] | undefined);
  useEffect(() => {
    let supervisor = new Supervisor();
    let model = loadAndSuperviseModel(supervisor);
    setVal([supervisor, model]);
    return () => setVal(undefined);
  }, []);

  if (val == undefined) {
    return <></>
  } else {
    return <SupervisorContext.Provider value = {val}>{children}</SupervisorContext.Provider>
  }
}

export function useModel(): Model {
  const val = useContext(SupervisorContext)
  if (val == undefined) {
    throw new Error("should never be undefined due to the structure of HasModel")
  }

  let [supervisor, model] = val;
  const [version, setVersion] = useState(0);
  useEffect(() => {
    let handle = supervisor.onNotify(() => setVersion(version + 1));
    return () => { supervisor.removeOnNotify(handle); }
  });
  return model
}