import {Model} from "./model.ts";
import {ActiveProject} from "./lair.ts";

export enum AgentRole {
  Vampire,
  Thrall
}

export type Agent = {
  index: number
  name: string
  activeTask: ActiveTask
}

export enum TaskType {
  Idle = 0,
  Project = 1
}

export type ActiveTask = {
  taskType: TaskType
  caption: string
  startedAt: number,
  remainingTurns: number
}

export class Agents {
  values: Array<Agent> = []

  get(agentId: number): Agent {
    return this.values[agentId];
  }

  generate(role: AgentRole): number {
    let id = this.values.length;
    let agent = {
      index: id,
      name: "Pyrex",
      activeTask: {
        taskType: TaskType.Idle,
        caption: "Idle",
        startedAt: 0,
        remainingTurns: 1
      }
    };
    this.values.push(agent);
    return id;
  }

  computeSchedule(model: Model) {
    // first of all: which agents have a project?
    let onProject: Record<number, [number, ActiveProject]> = {};
    for (let lair of model.lairs.values) {
      if (lair.activeProject) {
        if (lair.vampire != null) {
          onProject[lair.vampire] = [lair.index, lair.activeProject];
        }
        if (lair.thrall != null) {
          onProject[lair.thrall] = [lair.index, lair.activeProject];
        }
      }
    }

    // who has nothing to do?
    let unaccountedFor = [];
    for (let agent of this.values) {
      if (!onProject[agent.index]) {
        unaccountedFor.push(agent.index);
      }
    }

    for (let i of unaccountedFor) {
      this.transitionToIdle(model, i);
    }
    for (let agent of this.values) {
      if (onProject[agent.index]) {
        this.transitionToWork(model, agent.index, onProject[agent.index][1])
      }
    }
  }

  transitionToWork(model: Model, agent: number, project: ActiveProject) {
    let caption = project.verb;
    if (this.values[agent].activeTask.taskType == TaskType.Project) {
      this.values[agent].activeTask.caption = caption;
      this.values[agent].activeTask.remainingTurns = project.remainingTurns;
      return;
    }
    this.values[agent].activeTask = {
      taskType: TaskType.Project,
      caption: caption,
      startedAt: model.turn,
      remainingTurns: project.remainingTurns
    }
  }

  transitionToIdle(model: Model, agent: number) {
    this.values[agent].activeTask = {
      taskType: TaskType.Idle,
      caption: "Idle",
      startedAt: model.turn,
      remainingTurns: 1
    }
  }
}
