// == these are the room types ==
import {ResourceBundle} from "./resources.ts";
import {Model} from "./model.ts";

export class LairExtensions {
  extensions: Array<LairExtensionType> = []
  extensionObjects: Record<string, Array<LairExtensionObject>> = {}
  objectInteractions: Record<string, Array<LairExtensionInteraction>> = {}

  createExtension(extension: LairExtensionType): string {
    this.extensions.push(extension);
    return extension.id
  }

  addObjectToExtension(extensionId: string, object: LairExtensionObject): string {
    this.extensionObjects[extensionId] = this.extensionObjects[extensionId] ?? []
    this.extensionObjects[extensionId].push(object)
    return object.id;
  }

  addInteractionToObject(objectId: string, interaction: LairExtensionInteraction): string {
    this.objectInteractions[objectId] = this.objectInteractions[objectId] ?? []
    this.objectInteractions[objectId].push(interaction);
    return interaction.id
  }
}

export type LairExtensionType = {
  id: string,
  projectName: string,
  verb: string,
  extensionName: string,
  cost: ResourceBundle,
  turns: number,
  payoff: ((model: Model) => void) | null,
}
export type LairExtensionObject = {
  id: string,
  name: string,
  verb: string,
  cost: ResourceBundle,
  turns: number,
  payoff: ((model: Model) => void) | null,
}
export type LairExtensionInteraction = {
  id: string,
  name: string,
  verb: string,
  cost: ResourceBundle,
  turns: number,
  payoff: ((model: Model) => void) | null,
}

export function generateExtensions(_model: Model): LairExtensions {
  let extensions = new LairExtensions();

  const EXT_GROUNDS = extensions.createExtension({
    id: "grounds",
    projectName: "Renovate the grounds.",
    verb: "Renovating the grounds",
    extensionName: "Grounds",
    cost: {},
    turns: 5,
    payoff: null,
  });

  const OBJ_BOUNCY_CASTLE = extensions.addObjectToExtension(EXT_GROUNDS, {
    id: "bouncyCastle",
    name: "Build a bouncy castle.",
    verb: "Building a bouncy castle",
    cost: {},
    turns: 5,
    payoff: null
  })

  const INT_BOUNCE = extensions.addInteractionToObject(OBJ_BOUNCY_CASTLE, {
    id: "bounce",
    name: "Bounce in the bouncy castle.",
    verb: "Bouncing in the bouncy castle",
    cost: {},
    turns: 5,
    payoff: null,
  });

  return extensions
}

