import {Supervisor} from "./supervisor.ts";
import {Resources} from "./resources.ts";
import {Lairs, LairProject} from "./lair.ts";
import {NewLairPrice} from "./prices.ts";
import {Lifecycle} from "./lifecycle.ts";
import {AgentRole, Agents} from "./agent.ts";


export class Model {
  #supervisor?: Supervisor

  turn: number = 0
  lifecycle: Lifecycle = new Lifecycle()
  resources: Resources = new Resources()
  agents: Agents = new Agents()
  lairs: Lairs = new Lairs()

  constructor() {
    this.lifecycle.setupGameIfUnstarted(this);
  }

  setSupervisor(supervisor: Supervisor) {
    this.#supervisor = supervisor;
  }

  notify() {
    this.#supervisor?.notify()
  }

  nextTurn() {
    this.turn += 1
    this.lifecycle.recordTurn();
    this.lairs.updateProjects(this);
    this.agents.computeSchedule(this);
    this.notify()
  }

  buildNewLair() {
    if (!this.resources.spend(NewLairPrice)) { return; }

    let vampire = this.agents.generate(AgentRole.Vampire);
    let lair = this.lairs.generate();
    this.lairs.setVampire(lair, vampire);
    this.notify();
  }

  startWorkingOn(lair: number, project: LairProject) {
    this.lairs.startWorkingOn(lair, project)
    this.notify()
  }
}