export enum Resource {
  Gold, Blood, Enforcer, Thrall, Legend, Wisdom
}
export var resourceTypes: Array<Resource> = [Resource.Gold, Resource.Blood, Resource.Enforcer, Resource.Thrall, Resource.Legend, Resource.Wisdom];

export type ResourceBundle = {[t in Resource]?: number}

export class Resources {
  state: {[t in Resource]?: number} = {}

  get(resource: Resource): number {
    return this.state[resource] ?? 0
  }

  set(resource: Resource, amount: number) {
    this.state[resource] = amount
  }

  canSpend(bundle: ResourceBundle): boolean {
    for (let rt of resourceTypes) {
      if (this.get(rt) < (bundle[rt] ?? 0)) {
        return false;
      }
    }

    return true
  }

  receive(bundle: ResourceBundle) {
    for (let rt of resourceTypes) {
      this.set(rt, this.get(rt) + (bundle[rt] ?? 0));
    }
  }

  spend(bundle: ResourceBundle): boolean {
    if (!this.canSpend(bundle)) { return false; }
    for (let rt of resourceTypes) {
      this.set(rt, this.get(rt) - (bundle[rt] ?? 0));
    }
    return true;
  }
}