import { MutableRef } from "preact/hooks";
import {useEffect, useRef, useState} from "react";


export type ProgressBarCallback = () => {
  value: number | null,
  caption: string
}

export function ProgressBar({callback}: { callback: ProgressBarCallback }) {
  const canvasRef: MutableRef<HTMLCanvasElement | null> = useRef(null);
  const msgRef: MutableRef<HTMLDivElement | null> = useRef(null);
  const [text, setText] = useState("");
  const [enabled, setEnabled] = useState(false);

  useEffect(() => {
    let canceled: boolean = false
    let animate = () => {
      if (canceled || canvasRef.current == null) {
        return;
      }

      let canvas: HTMLCanvasElement = canvasRef.current;
      canvas.width = canvas.offsetWidth;
      canvas.height = 1
      let ctx = canvas.getContext("2d");

      if (ctx == null) {
        return;
      }

      let {value, caption} = callback();

      if (value == null) {
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        setEnabled(false);
      } else {
        ctx.fillStyle = "#000";
        ctx.fillRect(0, 0, canvas.width, canvas.height);
        ctx.fillStyle = "#f00";
        ctx.fillRect(0, 0, value * canvas.width, canvas.height);
        setEnabled(true);
      }

      setText(caption);
      requestAnimationFrame(animate);
    }
    requestAnimationFrame(animate);
    return () => { canceled = true; }
  })

  return <div class={"progressBar " + (enabled ? "enabled" : "disabled")}>
    <canvas ref={canvasRef} className={"backdrop"}></canvas>
    <div ref={msgRef} className={"caption small"}>{text}</div>
  </div>
}